//*****************************************************************************
//* Enkóder interfész modul.                                                  *
//* Rendszerarchitektúrák labor, 2. EDK-s mérés.                              *
//*****************************************************************************
module user_logic(
   //A PLB IPIF interfész portjai.
   Bus2IP_Clk,                     //Órajel.
   Bus2IP_Reset,                   //Reset jel.
   Bus2IP_Data,                    //Írási adatbusz (IPIF -> IP).
   Bus2IP_BE,                      //Bájt engedélyező jelek.
   Bus2IP_RdCE,                    //Regiszter olvasás engedélyező jelek.
   Bus2IP_WrCE,                    //Regiszter írás engedélyező jelek.
   IP2Bus_Data,                    //Olvasási adatbusz (IP -> IPIF).
   IP2Bus_RdAck,                   //Regiszter olvasás nyugtázó jel.
   IP2Bus_WrAck,                   //Regiszter írás nyugtázó jel.
   IP2Bus_Error,                   //Hibajelzés az IPIF felé.
   
   //Az enkóder interfész portjai.
   encoder_a,                      //Az enkóder A jele.
   encoder_b,                      //Az enkóder B jele.
   irq                             //Megszakításkérő kimenet.
);

//*****************************************************************************
//* A PLB IPIF interfész paraméterei (EZEKET NE MÓDOSÍTSA!).                  *
//*****************************************************************************
parameter C_SLV_DWIDTH = 32;
parameter C_NUM_REG    = 2;
parameter simulation   = 0;


//*****************************************************************************
//* A PLB IPIF interfész portjainak definiálása (EZEKET NE MÓDOSÍTSA!).       *
//*****************************************************************************
input  wire                        Bus2IP_Clk;
input  wire                        Bus2IP_Reset;
input  wire [0 : C_SLV_DWIDTH-1]   Bus2IP_Data;
input  wire [0 : C_SLV_DWIDTH/8-1] Bus2IP_BE;
input  wire [0 : C_NUM_REG-1]      Bus2IP_RdCE;
input  wire [0 : C_NUM_REG-1]      Bus2IP_WrCE;
output reg  [0 : C_SLV_DWIDTH-1]   IP2Bus_Data;
output wire                        IP2Bus_RdAck;
output wire                        IP2Bus_WrAck;
output wire                        IP2Bus_Error;


//*****************************************************************************
//* Az enkóder interfész portjainak definiálása (EZEKET NE MÓDOSÍTSA!).       *
//*****************************************************************************
input  wire                        encoder_a;
input  wire                        encoder_b;
output wire                        irq;


//*****************************************************************************
//* Az írási és olvasási adatbusz bitjeinek megfordítása.                     *
//* A továbbiakban a Bus2IP_Data jel helyett használja a wr_data jelet.       *
//* A továbbiakban a IP2Bus_Data jel helyett használja a rd_data jelet.       *
//*****************************************************************************
reg     [C_SLV_DWIDTH-1:0] wr_data;
reg     [C_SLV_DWIDTH-1:0] rd_data;
integer                    i;

always @(*)
   for (i = 0; i < C_SLV_DWIDTH; i = i + 1)
   begin
      wr_data[i] <= Bus2IP_Data[C_SLV_DWIDTH-i-1];
      IP2Bus_Data[i] <= rd_data[C_SLV_DWIDTH-i-1];
   end
      

//*****************************************************************************
//* A nyugtázó- és hibajelek meghajtása.                                      *
//*****************************************************************************
assign IP2Bus_WrAck = |Bus2IP_WrCE;
assign IP2Bus_RdAck = |Bus2IP_RdCE;
assign IP2Bus_Error = 1'b0;


//*****************************************************************************
//* Az enkóder jeleinek mintavételezése 1 kHz-el és az események dekódolása.  *
//*****************************************************************************

// órajel osztó (50000-rel oszt), 50 MHz -> 1 kHz
reg [15:0] div_cntr;
wire       div_cntr_max;

always @(posedge Bus2IP_Clk)
   if(Bus2IP_Reset || div_cntr_max)
      div_cntr <= 0;
   else
      div_cntr <= div_cntr + 1;

generate
   if(simulation == 0)
      assign div_cntr_max = (div_cntr == 49999);
   else
      assign div_cntr_max = (div_cntr == 3);      // szimulációhoz
endgenerate


reg [1:0] enc_a_shr;
reg [1:0] enc_b_shr;
wire      enc_a_rise;
wire      enc_b_rise;


// shift regiszter (felfutó él detektáláshoz)
always @ (posedge Bus2IP_Clk)
   if(Bus2IP_Reset)
      enc_a_shr <= 2'b00;
   else if(div_cntr_max)
      enc_a_shr <= {enc_a_shr[0], encoder_a};

always @ (posedge Bus2IP_Clk)
   if(Bus2IP_Reset)
      enc_b_shr <= 2'b00;
   else if(div_cntr_max)
      enc_b_shr <= {enc_b_shr[0], encoder_b};


// felfutó él detektálás
assign enc_a_rise = (enc_a_shr == 2'b01);
assign enc_b_rise = (enc_b_shr == 2'b01);


wire inc_cntr;
wire dec_cntr;


// növelo jel eloállítása: encoder_a = felfutó él és encoder_b = 0
assign inc_cntr = enc_a_rise && (enc_b_shr == 2'b00) && div_cntr_max;


// csökkento jel eloállítása: encoder_b = felfutó él és encoder_a = 0 
assign dec_cntr = enc_b_rise && (enc_a_shr == 2'b00) && div_cntr_max;


//*****************************************************************************
//* Vezérlő/státusz regiszter (BÁZIS+0x00).                                   *
//* A megszakításkérő jel előállítása.                                        *
//*****************************************************************************
reg [7:0]  enc_counter;    // encoder forgás számláló
reg        IE_bit;         // megszakítás engedélyzo
wire       IF_bit;         // megszakítás flag


always @ (posedge Bus2IP_Clk)
   if(Bus2IP_Reset)
      IE_bit <= 1'b0;
   else if (Bus2IP_WrCE[0] && (Bus2IP_BE == 4'b1111))
      IE_bit <= wr_data[0];


assign IF_bit = (enc_counter != 8'b0);
assign irq = IE_bit && IF_bit;

//*****************************************************************************
//* Számláló (BÁZIS+0x04).                                                    *
//*****************************************************************************
wire res_cntr;
assign res_cntr = Bus2IP_RdCE[1] && IP2Bus_RdAck;


// számláló léptetése
always @(posedge Bus2IP_Clk)
   if(Bus2IP_Reset)
      enc_counter <= 8'd0;
   else if(~res_cntr && inc_cntr)
      enc_counter <= enc_counter + 8'd1;
   else if(~res_cntr && dec_cntr)
      enc_counter <= enc_counter - 8'd1;
   else if(res_cntr && inc_cntr)
      enc_counter <= +8'd1;
   else if(res_cntr && dec_cntr)
      enc_counter <= -8'd1;
   else if(res_cntr)
      enc_counter <= 8'd0;
   else
      enc_counter <= enc_counter;


//*****************************************************************************
//* Az olvasási adatbusz meghajtása.                                          *
//*****************************************************************************

always @(*)
   case({Bus2IP_RdCE[1], Bus2IP_RdCE[0]})
      4'b01:   rd_data <= {30'b0, IF_bit, IE_bit};
      4'b10:   rd_data <= {{24{enc_counter[7]}}, enc_counter};
      default: rd_data <= 32'b0;
   endcase


endmodule
